@@ -52,7 +52,6 @@ use deno_runtime::deno_node::ops::ipc::ChildIpcSerialization;
 use deno_runtime::deno_permissions::AllowRunDescriptor;
 use deno_runtime::deno_permissions::PathDescriptor;
 use deno_runtime::deno_permissions::PermissionsOptions;
-use deno_runtime::inspector_server::InspectorServer;
 use deno_semver::StackString;
 use deno_semver::npm::NpmPackageReqReference;
 use deno_telemetry::OtelConfig;
@@ -448,13 +447,6 @@ impl WorkspaceMainModuleResolver {
               )?
               .into_url()?
           }
-          deno_package_json::PackageJsonDepValue::JsrReq(_) => {
-            return Err(
-              deno_resolver::DenoResolveErrorKind::UnsupportedPackageJsonJsrReq
-                .into_box()
-                .into(),
-            );
-          }
         }
       }
       deno_resolver::workspace::MappedResolution::PackageJsonImport {
@@ -833,23 +825,20 @@ impl CliOptions {
     self.workspace().vendor_dir_path()
   }
 
-  pub fn resolve_inspector_server(
+  pub fn resolve_inspector_server_options(
     &self,
-  ) -> Result<Option<InspectorServer>, AnyError> {
-    let maybe_inspect_host = self
+  ) -> Option<(SocketAddr, &'static str, InspectPublishUid)> {
+    let host = self
       .flags
       .inspect
       .or(self.flags.inspect_brk)
-      .or(self.flags.inspect_wait);
+      .or(self.flags.inspect_wait)?;
 
-    let Some(host) = maybe_inspect_host else {
-      return Ok(None);
-    };
-
-    Ok(Some(InspectorServer::new(
+    Some((
       host,
       DENO_VERSION_INFO.user_agent,
-    )?))
+      self.flags.inspect_publish_uid.unwrap_or_default(),
+    ))
   }
 
   pub fn resolve_fmt_options_for_members(
@@ -1023,12 +1012,12 @@ impl CliOptions {
       || self.flags.inspect_wait.is_some()
   }
 
-  pub fn inspect_brk(&self) -> Option<SocketAddr> {
-    self.flags.inspect_brk
+  pub fn inspect_brk(&self) -> bool {
+    self.flags.inspect_brk.is_some()
   }
 
-  pub fn inspect_wait(&self) -> Option<SocketAddr> {
-    self.flags.inspect_wait
+  pub fn inspect_wait(&self) -> bool {
+    self.flags.inspect_wait.is_some()
   }
 
   pub fn log_level(&self) -> Option<log::Level> {
@@ -1144,6 +1133,7 @@ impl CliOptions {
         "jsr.io:443",
         "deno.land:443",
         "esm.sh:443",
+        "raw.esm.sh:443",
         "cdn.jsdelivr.net:443",
         "raw.githubusercontent.com:443",
         "gist.githubusercontent.com:443",
@@ -1328,6 +1318,16 @@ impl CliOptions {
     unstable_features
   }
 
+  /// Returns unstable feature flags as CLI arguments (e.g., "--unstable-unsafe-proto").
+  /// This includes features from both CLI flags and config file.
+  pub fn unstable_args(&self) -> Vec<String> {
+    self
+      .unstable_features()
+      .into_iter()
+      .map(|f| format!("--unstable-{}", f))
+      .collect()
+  }
+
   pub fn v8_flags(&self) -> &Vec<String> {
     &self.flags.v8_flags
   }
@@ -1511,12 +1511,7 @@ pub fn config_to_deno_graph_workspace_member(
 pub fn get_default_v8_flags() -> Vec<String> {
   vec![
     "--stack-size=1024".to_string(),
-    "--js-explicit-resource-management".to_string(),
-    // TODO(bartlomieju): I think this can be removed as it's handled by `deno_core`
-    // and its settings.
-    // deno_ast removes TypeScript `assert` keywords, so this flag only affects JavaScript
-    // TODO(petamoriken): Need to check TypeScript `assert` keywords in deno_ast
-    "--no-harmony-import-assertions".to_string(),
+    "--inspector-live-edit".to_string(),
   ]
 }
 
