@@ -1049,6 +1049,7 @@ impl Config {
       | MediaType::Wasm
       | MediaType::Css
       | MediaType::Html
+      | MediaType::Markdown
       | MediaType::SourceMap
       | MediaType::Sql
       | MediaType::Unknown => None,
@@ -1079,7 +1080,7 @@ impl Config {
   }
 
   pub fn uri_enabled(&self, uri: &Uri) -> bool {
-    if uri.scheme().is_some_and(|s| s.eq_lowercase("deno")) {
+    if uri.scheme().as_str().eq_ignore_ascii_case("deno") {
       return true;
     }
     self.specifier_enabled(&uri_to_url(uri))
@@ -1179,7 +1180,7 @@ impl Config {
   pub fn diagnostic_refresh_capable(&self) -> bool {
     (|| {
       let workspace = self.client_capabilities.workspace.as_ref()?;
-      workspace.diagnostic.as_ref()?.refresh_support
+      workspace.diagnostics.as_ref()?.refresh_support
     })()
     .unwrap_or(false)
   }
@@ -1486,6 +1487,7 @@ impl ConfigData {
       pb,
       None,
       NpmInstallerFactoryOptions {
+        clean_on_install: false,
         cache_setting: NpmCacheSetting::Use,
         caching_strategy: NpmCachingStrategy::Eager,
         lifecycle_scripts_config: LifecycleScriptsConfig::default(),
