@@ -9,10 +9,12 @@ use std::sync::Arc;
 
 use deno_ast::MediaType;
 use deno_ast::ModuleSpecifier;
+use deno_config::deno_json::NodeModulesDirMode;
 use deno_core::anyhow::Context;
 use deno_core::anyhow::anyhow;
 use deno_core::anyhow::bail;
 use deno_core::error::AnyError;
+use deno_core::futures::FutureExt;
 use deno_graph::GraphKind;
 use deno_npm_installer::graph::NpmCachingStrategy;
 use deno_path_util::resolve_url_or_path;
@@ -24,12 +26,44 @@ use rand::Rng;
 use super::installer::BinNameResolver;
 use crate::args::CliOptions;
 use crate::args::CompileFlags;
+use crate::args::ConfigFlag;
 use crate::args::Flags;
 use crate::factory::CliFactory;
 use crate::standalone::binary::WriteBinOptions;
 use crate::standalone::binary::is_standalone_binary;
+use crate::util::temp::create_temp_node_modules_dir;
 
 pub async fn compile(
+  mut flags: Flags,
+  compile_flags: CompileFlags,
+) -> Result<(), AnyError> {
+  // use a temporary directory with a node_modules folder when the user
+  // specifies an npm package for better compatibility
+  let _temp_dir =
+    if compile_flags.source_file.to_lowercase().starts_with("npm:")
+      && flags.node_modules_dir.is_none()
+      && !matches!(flags.config_flag, ConfigFlag::Path(_))
+    {
+      let temp_node_modules_dir = create_temp_node_modules_dir()
+        .context("Failed creating temp directory for node_modules folder.")?;
+      flags.initial_cwd = Some(temp_node_modules_dir.parent().to_path_buf());
+      flags.internal.root_node_modules_dir_override =
+        Some(temp_node_modules_dir.node_modules_dir_path().to_path_buf());
+      flags.node_modules_dir = Some(NodeModulesDirMode::Auto);
+      Some(temp_node_modules_dir)
+    } else {
+      None
+    };
+  let flags = Arc::new(flags);
+  // boxed_local() is to avoid large futures
+  if compile_flags.eszip {
+    compile_eszip(flags, compile_flags).boxed_local().await
+  } else {
+    compile_binary(flags, compile_flags).boxed_local().await
+  }
+}
+
+async fn compile_binary(
   flags: Arc<Flags>,
   compile_flags: CompileFlags,
 ) -> Result<(), AnyError> {
@@ -172,7 +206,7 @@ pub async fn compile(
   Ok(())
 }
 
-pub async fn compile_eszip(
+async fn compile_eszip(
   flags: Arc<Flags>,
   compile_flags: CompileFlags,
 ) -> Result<(), AnyError> {
@@ -372,6 +406,7 @@ fn get_module_roots_and_include_paths(
       | MediaType::Html
       | MediaType::Jsonc
       | MediaType::Json5
+      | MediaType::Markdown
       | MediaType::SourceMap
       | MediaType::Sql
       | MediaType::Unknown => false,
