@@ -11,9 +11,10 @@ use deno_config::glob::PathOrPattern;
 use deno_config::glob::PathOrPatternSet;
 use deno_config::glob::WalkEntry;
 use deno_core::ModuleSpecifier;
-use deno_core::anyhow::Context;
 use deno_core::anyhow::anyhow;
 use deno_core::error::AnyError;
+use sys_traits::FsMetadataValue;
+use sys_traits::PathsInErrorsExt;
 
 use super::progress_bar::UpdateGuard;
 use crate::sys::CliSys;
@@ -195,14 +196,16 @@ impl FsCleaner {
     }
   }
 
-  pub fn rm_rf(&mut self, path: &Path) -> Result<(), AnyError> {
+  pub fn rm_rf(&mut self, path: &Path) -> Result<(), std::io::Error> {
+    let sys = CliSys::default();
+    let sys = sys.with_paths_in_errors();
     for entry in walkdir::WalkDir::new(path).contents_first(true) {
-      let entry = entry?;
+      let entry = entry.map_err(std::io::Error::other)?;
 
       if entry.file_type().is_dir() {
         self.dirs_removed += 1;
         self.update_progress();
-        std::fs::remove_dir_all(entry.path())?;
+        sys.fs_remove_dir_all(entry.path())?;
       } else {
         self.remove_file(entry.path(), entry.metadata().ok())?;
       }
@@ -215,29 +218,13 @@ impl FsCleaner {
     &mut self,
     path: &Path,
     meta: Option<std::fs::Metadata>,
-  ) -> Result<(), AnyError> {
+  ) -> Result<(), std::io::Error> {
     if let Some(meta) = meta {
       self.bytes_removed += meta.len();
     }
     self.files_removed += 1;
     self.update_progress();
-    match std::fs::remove_file(path)
-      .with_context(|| format!("Failed to remove file: {}", path.display()))
-    {
-      Err(e) => {
-        if cfg!(windows)
-          && let Ok(meta) = path.symlink_metadata()
-          && meta.is_symlink()
-        {
-          std::fs::remove_dir(path).with_context(|| {
-            format!("Failed to remove symlink: {}", path.display())
-          })?;
-          return Ok(());
-        }
-        Err(e)
-      }
-      _ => Ok(()),
-    }
+    remove_file_or_symlink(&CliSys::default(), path)
   }
 
   fn update_progress(&self) {
@@ -247,6 +234,30 @@ impl FsCleaner {
   }
 }
 
+pub fn remove_file_or_symlink<
+  TSys: sys_traits::BaseFsRemoveFile
+    + sys_traits::BaseFsRemoveDir
+    + sys_traits::BaseFsMetadata,
+>(
+  sys: &TSys,
+  path: &Path,
+) -> Result<(), Error> {
+  let sys = sys.with_paths_in_errors();
+  match sys.fs_remove_file(path) {
+    Err(e) => {
+      if sys_traits::impls::is_windows()
+        && let Ok(meta) = sys.fs_symlink_metadata(path)
+        && meta.file_type().is_symlink()
+      {
+        sys.fs_remove_dir(path)?;
+        return Ok(());
+      }
+      Err(e)
+    }
+    _ => Ok(()),
+  }
+}
+
 #[cfg(test)]
 mod tests {
   use pretty_assertions::assert_eq;
