@@ -31,7 +31,6 @@ use cache_control::CacheControl;
 use deno_core::AsyncRefCell;
 use deno_core::AsyncResult;
 use deno_core::BufView;
-use deno_core::ByteString;
 use deno_core::CancelFuture;
 use deno_core::CancelHandle;
 use deno_core::CancelTryFuture;
@@ -41,6 +40,7 @@ use deno_core::RcRef;
 use deno_core::Resource;
 use deno_core::ResourceId;
 use deno_core::StringOrBuffer;
+use deno_core::convert::ByteString;
 use deno_core::futures::FutureExt;
 use deno_core::futures::StreamExt;
 use deno_core::futures::TryFutureExt;
@@ -78,7 +78,6 @@ use hyper_v014::header::HeaderValue;
 use hyper_v014::server::conn::Http;
 use hyper_v014::service::Service;
 use once_cell::sync::OnceCell;
-use serde::Serialize;
 use tokio::io::AsyncRead;
 use tokio::io::AsyncWrite;
 use tokio::io::AsyncWriteExt;
@@ -994,8 +993,7 @@ impl Drop for BodyUncompressedSender {
 }
 
 // We use a tuple instead of struct to avoid serialization overhead of the keys.
-#[derive(Serialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(deno_core::ToV8)]
 struct NextRequestResponse(
   // read_stream_rid:
   ResourceId,
@@ -1009,8 +1007,7 @@ struct NextRequestResponse(
   String,
 );
 
-#[op2(async)]
-#[serde]
+#[op2]
 async fn op_http_accept(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -1119,12 +1116,12 @@ fn req_headers(
   headers
 }
 
-#[op2(async)]
+#[op2]
 async fn op_http_write_headers(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: u32,
   #[smi] status: u16,
-  #[serde] headers: Vec<(ByteString, ByteString)>,
+  #[scoped] headers: Vec<(ByteString, ByteString)>,
   #[serde] data: Option<StringOrBuffer>,
 ) -> Result<(), HttpError> {
   let stream = state
@@ -1198,7 +1195,6 @@ async fn op_http_write_headers(
 }
 
 #[op2]
-#[serde]
 fn op_http_headers(
   state: &mut OpState,
   #[smi] rid: u32,
@@ -1351,7 +1347,7 @@ fn should_compress(headers: &hyper_v014::HeaderMap) -> bool {
       .unwrap_or_default()
 }
 
-#[op2(async)]
+#[op2]
 async fn op_http_write_resource(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -1410,7 +1406,7 @@ async fn op_http_write_resource(
   Ok(())
 }
 
-#[op2(async)]
+#[op2]
 async fn op_http_write(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -1472,7 +1468,7 @@ async fn op_http_write(
 /// Gracefully closes the write half of the HTTP stream. Note that this does not
 /// remove the HTTP stream resource from the resource table; it still has to be
 /// closed with `Deno.core.close()`.
-#[op2(async)]
+#[op2]
 async fn op_http_shutdown(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -1517,7 +1513,7 @@ fn op_http_websocket_accept_header(#[string] key: String) -> String {
   BASE64_STANDARD.encode(digest)
 }
 
-#[op2(async)]
+#[op2]
 #[smi]
 async fn op_http_upgrade_websocket(
   state: Rc<RefCell<OpState>>,
@@ -1702,7 +1698,6 @@ fn extract_network_stream<U: CanDowncastUpgrade>(
 }
 
 #[op2]
-#[serde]
 pub fn op_http_serve_address_override() -> (u8, String, u32, bool) {
   if let Ok(val) = std::env::var("DENO_SERVE_ADDRESS") {
     return parse_serve_address(&val);
