@@ -25,7 +25,8 @@ use deno_core::OpState;
 use deno_core::RcRef;
 use deno_core::Resource;
 use deno_core::ResourceId;
-use deno_core::ToJsBuffer;
+use deno_core::ToV8;
+use deno_core::convert::Uint8Array;
 use deno_core::op2;
 use deno_core::serde_json;
 use deno_error::JsErrorBox;
@@ -45,7 +46,6 @@ use deno_subprocess_windows::Command;
 #[cfg(windows)]
 use deno_subprocess_windows::Stdio as StdStdio;
 use serde::Deserialize;
-use serde::Serialize;
 #[cfg(unix)]
 use tokio::process::Child as AsyncChild;
 
@@ -313,8 +313,7 @@ pub struct ChildStdio {
   stderr: StdioOrRid,
 }
 
-#[derive(Serialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(ToV8)]
 pub struct ChildStatus {
   success: bool,
   code: i32,
@@ -354,12 +353,11 @@ impl TryFrom<ExitStatus> for ChildStatus {
   }
 }
 
-#[derive(Serialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(ToV8)]
 pub struct SpawnOutput {
   status: ChildStatus,
-  stdout: Option<ToJsBuffer>,
-  stderr: Option<ToJsBuffer>,
+  stdout: Option<Uint8Array>,
+  stderr: Option<Uint8Array>,
 }
 
 type CreateCommand = (
@@ -652,8 +650,7 @@ fn create_command(
   }
 }
 
-#[derive(Serialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(ToV8)]
 struct Child {
   rid: ResourceId,
   pid: u32,
@@ -1018,7 +1015,6 @@ fn get_requires_allow_all_env_vars(env: &RunEnv) -> Vec<&str> {
 }
 
 #[op2(stack_trace)]
-#[serde]
 fn op_spawn_child(
   state: &mut OpState,
   #[serde] args: SpawnArgs,
@@ -1034,9 +1030,8 @@ fn op_spawn_child(
   child
 }
 
-#[op2(async)]
+#[op2]
 #[allow(clippy::await_holding_refcell_ref)]
-#[serde]
 async fn op_spawn_wait(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -1060,7 +1055,6 @@ async fn op_spawn_wait(
 }
 
 #[op2(stack_trace)]
-#[serde]
 fn op_spawn_sync(
   state: &mut OpState,
   #[serde] args: SpawnArgs,
@@ -1125,6 +1119,7 @@ fn op_spawn_kill(
 }
 
 mod deprecated {
+  use deno_core::FromV8;
   #[cfg(windows)]
   use deno_subprocess_windows::Child;
   #[cfg(not(windows))]
@@ -1132,14 +1127,16 @@ mod deprecated {
 
   use super::*;
 
-  #[derive(Deserialize)]
-  #[serde(rename_all = "camelCase")]
+  #[derive(FromV8)]
   pub struct RunArgs {
     cmd: Vec<String>,
     cwd: Option<String>,
     env: Vec<(String, String)>,
+    #[from_v8(serde)]
     stdin: StdioOrRid,
+    #[from_v8(serde)]
     stdout: StdioOrRid,
+    #[from_v8(serde)]
     stderr: StdioOrRid,
   }
 
@@ -1159,8 +1156,7 @@ mod deprecated {
     }
   }
 
-  #[derive(Serialize)]
-  #[serde(rename_all = "camelCase")]
+  #[derive(ToV8)]
   // TODO(@AaronO): maybe find a more descriptive name or a convention for return structs
   pub struct RunInfo {
     rid: ResourceId,
@@ -1171,10 +1167,9 @@ mod deprecated {
   }
 
   #[op2(stack_trace)]
-  #[serde]
   pub fn op_run(
     state: &mut OpState,
-    #[serde] run_args: RunArgs,
+    #[scoped] run_args: RunArgs,
   ) -> Result<RunInfo, ProcessError> {
     let args = run_args.cmd;
     let cmd = args.first().ok_or(ProcessError::MissingCmd)?;
@@ -1285,16 +1280,14 @@ mod deprecated {
     })
   }
 
-  #[derive(Serialize)]
-  #[serde(rename_all = "camelCase")]
+  #[derive(ToV8)]
   pub struct ProcessStatus {
     got_signal: bool,
     exit_code: i32,
     exit_signal: i32,
   }
 
-  #[op2(async)]
-  #[serde]
+  #[op2]
   pub async fn op_run_status(
     state: Rc<RefCell<OpState>>,
     #[smi] rid: ResourceId,
