@@ -8,15 +8,14 @@ use std::rc::Rc;
 use bytes::Bytes;
 use deno_core::AsyncMutFuture;
 use deno_core::AsyncRefCell;
-use deno_core::ByteString;
 use deno_core::CancelHandle;
 use deno_core::CancelTryFuture;
-use deno_core::JsBuffer;
 use deno_core::OpState;
 use deno_core::RcRef;
 use deno_core::Resource;
 use deno_core::ResourceId;
-use deno_core::ToJsBuffer;
+use deno_core::convert::ByteString;
+use deno_core::convert::Uint8Array;
 use deno_core::futures::TryFutureExt;
 use deno_core::op2;
 use deno_core::unsync::spawn;
@@ -51,7 +50,6 @@ use http::header::SEC_WEBSOCKET_VERSION;
 use http::header::UPGRADE;
 use hyper_util::client::legacy::connect::Connection;
 use once_cell::sync::Lazy;
-use serde::Serialize;
 use tokio::io::AsyncRead;
 use tokio::io::AsyncWrite;
 use tokio::io::ReadHalf;
@@ -140,8 +138,7 @@ pub fn op_ws_check_permission_and_cancel_handle(
   }
 }
 
-#[derive(Serialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(deno_core::ToV8)]
 pub struct CreateResponse {
   rid: ResourceId,
   protocol: String,
@@ -397,15 +394,14 @@ fn populate_common_request_headers(
   Ok(request)
 }
 
-#[op2(async, stack_trace)]
-#[serde]
+#[op2(stack_trace)]
 pub async fn op_ws_create(
   state: Rc<RefCell<OpState>>,
   #[string] api_name: String,
   #[string] url: String,
   #[string] protocols: String,
   #[smi] cancel_handle: Option<ResourceId>,
-  #[serde] headers: Option<Vec<(ByteString, ByteString)>>,
+  #[scoped] headers: Option<Vec<(ByteString, ByteString)>>,
   #[smi] client_rid: Option<u32>,
 ) -> Result<CreateResponse, WebsocketError> {
   let (client, allow_host) = {
@@ -632,25 +628,28 @@ pub fn op_ws_send_text(
 }
 
 /// Async version of send. Does not update buffered amount as we rely on the socket itself for backpressure.
-#[op2(async)]
+#[op2]
 pub async fn op_ws_send_binary_async(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
-  #[buffer] data: JsBuffer,
+  data: Uint8Array,
 ) -> Result<(), WebsocketError> {
   let resource = state
     .borrow_mut()
     .resource_table
     .get::<ServerWebSocket>(rid)?;
-  let data = data.to_vec();
+  let data = data.0;
   let lock = resource.reserve_lock();
   resource
-    .write_frame(lock, Frame::new(true, OpCode::Binary, None, data.into()))
+    .write_frame(
+      lock,
+      Frame::new(true, OpCode::Binary, None, (&*data).into()),
+    )
     .await
 }
 
 /// Async version of send. Does not update buffered amount as we rely on the socket itself for backpressure.
-#[op2(async)]
+#[op2]
 pub async fn op_ws_send_text_async(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -685,7 +684,7 @@ pub fn op_ws_get_buffered_amount(
     .get() as u32
 }
 
-#[op2(async)]
+#[op2]
 pub async fn op_ws_send_ping(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
@@ -721,7 +720,10 @@ pub async fn op_ws_close(
   const EMPTY_PAYLOAD: &[u8] = &[];
 
   let frame = reason
-    .map(|reason| Frame::close(code.unwrap_or(1005), reason.as_bytes()))
+    .map(|reason| match code {
+      Some(code) => Frame::close(code, &reason.into_bytes()),
+      _ => Frame::close_raw(reason.into_bytes().into()),
+    })
     .unwrap_or_else(|| match code {
       Some(code) => Frame::close(code, EMPTY_PAYLOAD),
       _ => Frame::close_raw(EMPTY_PAYLOAD.into()),
@@ -733,15 +735,14 @@ pub async fn op_ws_close(
 }
 
 #[op2]
-#[serde]
 pub fn op_ws_get_buffer(
   state: &mut OpState,
   #[smi] rid: ResourceId,
-) -> Option<ToJsBuffer> {
+) -> Option<Uint8Array> {
   let Ok(resource) = state.resource_table.get::<ServerWebSocket>(rid) else {
     return None;
   };
-  resource.buffer.take().map(ToJsBuffer::from)
+  resource.buffer.take().map(Uint8Array::from)
 }
 
 #[op2]
@@ -766,7 +767,7 @@ pub fn op_ws_get_error(state: &mut OpState, #[smi] rid: ResourceId) -> String {
   resource.error.take().unwrap_or_default()
 }
 
-#[op2(async)]
+#[op2]
 pub async fn op_ws_next_event(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
