@@ -417,6 +417,9 @@ pub struct DenoGraphLoader<
   THttpClient: HttpClient,
 > {
   file_header_overrides: HashMap<Url, HashMap<String, String>>,
+  // Arc is ok because this is for the source
+  #[allow(clippy::disallowed_types)]
+  file_content_overrides: Option<HashMap<Url, std::sync::Arc<[u8]>>>,
   file_fetcher: PermissionedFileFetcherRc<TBlobStore, TSys, THttpClient>,
   global_http_cache: GlobalHttpCacheRc<TSys>,
   in_npm_pkg_checker: DenoInNpmPackageChecker,
@@ -448,9 +451,19 @@ impl<
       permissions: options.permissions,
       cache_info_enabled: false,
       reporter: options.reporter,
+      file_content_overrides: Default::default(),
     }
   }
 
+  // Arc is ok because this is for the source
+  #[allow(clippy::disallowed_types)]
+  pub fn set_file_content_overrides(
+    &mut self,
+    overrides: HashMap<Url, std::sync::Arc<[u8]>>,
+  ) {
+    self.file_content_overrides = Some(overrides);
+  }
+
   pub fn insert_file_header_override(
     &mut self,
     specifier: Url,
@@ -639,6 +652,17 @@ impl<
         },
       ))));
     }
+    if let Some(overrides) = &self.file_content_overrides
+      && let Some(content) = overrides.get(specifier)
+    {
+      return std::future::ready(Ok(Some(LoadResponse::Module {
+        content: content.clone(),
+        mtime: None,
+        specifier: specifier.clone(),
+        maybe_headers: self.file_header_overrides.get(specifier).cloned(),
+      })))
+      .boxed_local();
+    }
 
     self.load_or_cache(
       LoadStrategy {
@@ -656,6 +680,15 @@ impl<
     specifier: &Url,
     options: deno_graph::source::LoadOptions,
   ) -> deno_graph::source::EnsureCachedFuture {
+    if let Some(overrides) = &self.file_content_overrides
+      && overrides.contains_key(specifier)
+    {
+      return std::future::ready(Ok(Some(
+        deno_graph::source::CacheResponse::Cached,
+      )))
+      .boxed_local();
+    }
+
     self.load_or_cache(
       CacheStrategy {
         file_fetcher: self.file_fetcher.clone(),
@@ -784,3 +817,191 @@ fn validate_scheme(specifier: &Url) -> Result<(), UnsupportedSchemeError> {
     }),
   }
 }
+
+#[cfg(test)]
+mod test {
+  use std::collections::HashMap;
+  use std::path::PathBuf;
+
+  use deno_cache_dir::file_fetcher::CacheSetting;
+  use deno_cache_dir::file_fetcher::NullBlobStore;
+  use deno_cache_dir::file_fetcher::SendError;
+  use deno_cache_dir::file_fetcher::SendResponse;
+  use deno_graph::source::CacheSetting as LoaderCacheSetting;
+  use deno_graph::source::LoadResponse;
+  use deno_graph::source::Loader;
+  use sys_traits::impls::InMemorySys;
+  use url::Url;
+
+  use super::*;
+  use crate::factory::ConfigDiscoveryOption;
+  use crate::factory::WorkspaceFactory;
+  use crate::factory::WorkspaceFactoryOptions;
+
+  #[derive(Debug)]
+  struct TestHttpClient;
+
+  #[async_trait::async_trait(?Send)]
+  impl deno_cache_dir::file_fetcher::HttpClient for TestHttpClient {
+    async fn send_no_follow(
+      &self,
+      _url: &Url,
+      _headers: http::HeaderMap,
+    ) -> Result<SendResponse, SendError> {
+      Err(SendError::NotFound)
+    }
+  }
+
+  fn create_test_loader(
+    sys: InMemorySys,
+  ) -> DenoGraphLoader<NullBlobStore, InMemorySys, TestHttpClient> {
+    let cwd = get_cwd();
+    let factory = WorkspaceFactory::new(
+      sys.clone(),
+      cwd.join("project"),
+      WorkspaceFactoryOptions {
+        maybe_custom_deno_dir_root: Some(cwd.join("deno_dir")),
+        config_discovery: ConfigDiscoveryOption::Disabled,
+        ..Default::default()
+      },
+    );
+    let global_http_cache = factory.global_http_cache().unwrap().clone();
+    let memory_files =
+      deno_maybe_sync::new_rc(crate::loader::MemoryFiles::default());
+    let file_fetcher = deno_maybe_sync::new_rc(PermissionedFileFetcher::new(
+      NullBlobStore,
+      deno_maybe_sync::new_rc(deno_cache_dir::GlobalOrLocalHttpCache::from(
+        global_http_cache.clone(),
+      )),
+      TestHttpClient,
+      memory_files,
+      sys.clone(),
+      PermissionedFileFetcherOptions {
+        allow_remote: false,
+        cache_setting: CacheSetting::Use,
+      },
+    ));
+    DenoGraphLoader::new(
+      file_fetcher,
+      global_http_cache,
+      crate::npm::DenoInNpmPackageChecker::new(
+        crate::npm::CreateInNpmPkgCheckerOptions::Byonm,
+      ),
+      sys,
+      DenoGraphLoaderOptions {
+        file_header_overrides: HashMap::new(),
+        permissions: None,
+        reporter: None,
+      },
+    )
+  }
+
+  fn load_options() -> deno_graph::source::LoadOptions {
+    deno_graph::source::LoadOptions {
+      in_dynamic_branch: false,
+      was_dynamic_root: false,
+      cache_setting: LoaderCacheSetting::Use,
+      maybe_checksum: None,
+    }
+  }
+
+  #[test]
+  fn file_content_overrides_load_returns_overridden_content() {
+    let cwd = get_cwd();
+    let sys = InMemorySys::new_with_cwd(&cwd);
+    let mut loader = create_test_loader(sys);
+
+    let specifier =
+      deno_path_util::url_from_file_path(&cwd.join("test.ts")).unwrap();
+    let content = b"console.log('hello')".as_slice().into();
+    let mut overrides = HashMap::new();
+    overrides.insert(specifier.clone(), content);
+    loader.set_file_content_overrides(overrides);
+
+    let mut pool = futures::executor::LocalPool::new();
+    let result = pool
+      .run_until(loader.load(&specifier, load_options()))
+      .unwrap();
+
+    match result {
+      Some(LoadResponse::Module {
+        content: loaded_content,
+        specifier: loaded_specifier,
+        mtime,
+        maybe_headers,
+      }) => {
+        assert_eq!(loaded_specifier, specifier);
+        assert_eq!(&*loaded_content, b"console.log('hello')");
+        assert!(mtime.is_none());
+        assert!(maybe_headers.is_none());
+      }
+      other => panic!("expected Module response, got {:?}", other),
+    }
+  }
+
+  #[test]
+  fn file_content_overrides_load_includes_header_overrides() {
+    let cwd = get_cwd();
+    let sys = InMemorySys::new_with_cwd(&cwd);
+    let mut loader = create_test_loader(sys);
+
+    let specifier =
+      deno_path_util::url_from_file_path(&cwd.join("test.ts")).unwrap();
+    let content = b"export {}".as_slice().into();
+    let mut overrides = HashMap::new();
+    overrides.insert(specifier.clone(), content);
+    loader.set_file_content_overrides(overrides);
+
+    let mut headers = HashMap::new();
+    headers.insert(
+      "content-type".to_string(),
+      "application/typescript".to_string(),
+    );
+    loader.insert_file_header_override(specifier.clone(), headers);
+
+    let mut pool = futures::executor::LocalPool::new();
+    let result = pool
+      .run_until(loader.load(&specifier, load_options()))
+      .unwrap();
+
+    match result {
+      Some(LoadResponse::Module { maybe_headers, .. }) => {
+        let h = maybe_headers.unwrap();
+        assert_eq!(h.get("content-type").unwrap(), "application/typescript");
+      }
+      other => panic!("expected Module response, got {:?}", other),
+    }
+  }
+
+  #[test]
+  fn file_content_overrides_ensure_cached_returns_cached() {
+    let cwd = get_cwd();
+    let sys = InMemorySys::new_with_cwd(&cwd);
+    let mut loader = create_test_loader(sys);
+
+    let specifier =
+      deno_path_util::url_from_file_path(&cwd.join("test.ts")).unwrap();
+    let content = b"export {}".as_slice().into();
+    let mut overrides = HashMap::new();
+    overrides.insert(specifier.clone(), content);
+    loader.set_file_content_overrides(overrides);
+
+    let mut pool = futures::executor::LocalPool::new();
+    let result = pool
+      .run_until(loader.ensure_cached(&specifier, load_options()))
+      .unwrap();
+
+    assert!(matches!(
+      result,
+      Some(deno_graph::source::CacheResponse::Cached)
+    ));
+  }
+
+  fn get_cwd() -> PathBuf {
+    if cfg!(windows) {
+      PathBuf::from("K:\\folder\\")
+    } else {
+      PathBuf::from("/")
+    }
+  }
+}
