@@ -84,7 +84,6 @@ pub struct DenoResolveError(pub Box<DenoResolveErrorKind>);
 impl DenoResolveError {
   #[cfg(feature = "graph")]
   pub fn into_deno_graph_error(self) -> deno_graph::source::ResolveError {
-    use deno_error::JsErrorBox;
     use deno_graph::source::ResolveError;
 
     match self.into_kind() {
@@ -92,16 +91,14 @@ impl DenoResolveError {
         match mapped_resolution_error {
           MappedResolutionError::Specifier(e) => ResolveError::Specifier(e),
           // deno_graph checks specifically for an ImportMapError
-          MappedResolutionError::ImportMap(e) => ResolveError::ImportMap(e),
-          MappedResolutionError::Workspace(e) => {
-            ResolveError::Other(JsErrorBox::from_err(e))
-          }
+          MappedResolutionError::ImportMap(e) => ResolveError::from_err(e),
+          MappedResolutionError::Workspace(e) => ResolveError::from_err(e),
           MappedResolutionError::NotFoundInCompilerOptionsPaths(e) => {
-            ResolveError::Other(JsErrorBox::from_err(e))
+            ResolveError::from_err(e)
           }
         }
       }
-      err => ResolveError::Other(JsErrorBox::from_err(err)),
+      err => ResolveError::from_err(err),
     }
   }
 
@@ -117,7 +114,6 @@ impl DenoResolveError {
       | DenoResolveErrorKind::ResolvePkgFolderFromDenoReq(_)
       | DenoResolveErrorKind::InvalidVendorFolderImport
       | DenoResolveErrorKind::UnsupportedPackageJsonFileSpecifier
-      | DenoResolveErrorKind::UnsupportedPackageJsonJsrReq
       | DenoResolveErrorKind::NodeModulesOutOfDate(_)
       | DenoResolveErrorKind::PackageJsonDepValueParse(_)
       | DenoResolveErrorKind::PackageJsonDepValueUrlParse(_) => None,
@@ -137,9 +133,6 @@ pub enum DenoResolveErrorKind {
     "Importing npm packages via a file: specifier is only supported with --node-modules-dir=manual"
   )]
   UnsupportedPackageJsonFileSpecifier,
-  #[class(type)]
-  #[error("JSR specifiers are not yet supported in package.json")]
-  UnsupportedPackageJsonJsrReq,
   #[class(inherit)]
   #[error(transparent)]
   MappedResolution(#[from] MappedResolutionError),
@@ -174,7 +167,6 @@ impl DenoResolveErrorKind {
     match self {
       DenoResolveErrorKind::InvalidVendorFolderImport
       | DenoResolveErrorKind::UnsupportedPackageJsonFileSpecifier
-      | DenoResolveErrorKind::UnsupportedPackageJsonJsrReq
       | DenoResolveErrorKind::MappedResolution { .. }
       | DenoResolveErrorKind::NodeModulesOutOfDate { .. }
       | DenoResolveErrorKind::PackageJsonDepValueParse { .. }
@@ -424,9 +416,6 @@ impl<
                     .into_box(),
                 )
               }
-              PackageJsonDepValue::JsrReq(_) => Err(
-                DenoResolveErrorKind::UnsupportedPackageJsonJsrReq.into_box(),
-              ),
               // todo(dsherret): it seems bad that we're converting this
               // to a url because the req might not be a valid url.
               PackageJsonDepValue::Req(req) => Url::parse(&format!(
