@@ -26,21 +26,13 @@ use crate::emit::EmitParsedSourceHelperError;
 use crate::emit::EmitterRc;
 use crate::factory::DenoNodeCodeTranslatorRc;
 use crate::graph::EnhanceGraphErrorMode;
+use crate::graph::EnhancedGraphError;
 use crate::graph::enhance_graph_error;
 use crate::npm::DenoInNpmPackageChecker;
 
 #[allow(clippy::disallowed_types)]
 type ArcStr = std::sync::Arc<str>;
 
-#[derive(Debug, thiserror::Error, deno_error::JsError)]
-#[error("{message}")]
-#[class(inherit)]
-pub struct EnhancedGraphError {
-  #[inherit]
-  pub error: deno_graph::ModuleError,
-  pub message: String,
-}
-
 #[derive(Debug, deno_error::JsError, Boxed)]
 #[class(inherit)]
 pub struct LoadPreparedModuleError(pub Box<LoadPreparedModuleErrorKind>);
@@ -398,15 +390,13 @@ impl<TSys: ModuleLoaderSys> PreparedModuleLoader<TSys> {
     specifier: &Url,
     requested_module_type: &RequestedModuleType,
   ) -> Result<Option<CodeOrDeferredEmit<'graph>>, LoadPreparedModuleError> {
-    let maybe_module =
-      graph.try_get(specifier).map_err(|err| EnhancedGraphError {
-        message: enhance_graph_error(
-          &self.sys,
-          &deno_graph::ModuleGraphError::ModuleError(err.clone()),
-          EnhanceGraphErrorMode::ShowRange,
-        ),
-        error: err.clone(),
-      })?;
+    let maybe_module = graph.try_get(specifier).map_err(|err| {
+      enhance_graph_error(
+        &self.sys,
+        deno_graph::ModuleGraphError::ModuleError(err.clone()),
+        EnhanceGraphErrorMode::ShowRange,
+      )
+    })?;
 
     match maybe_module {
       Some(deno_graph::Module::Json(JsonModule {
@@ -499,6 +489,7 @@ impl<TSys: ModuleLoaderSys> PreparedModuleLoader<TSys> {
             | MediaType::Html
             | MediaType::Jsonc
             | MediaType::Json5
+            | MediaType::Markdown
             | MediaType::Sql
             | MediaType::Wasm
             | MediaType::SourceMap => {
