@@ -299,13 +299,6 @@ impl<TSys: LockfileSys> LockfileLock<TSys> {
           PackageJsonDepValue::Req(req) => {
             Some(JsrDepPackageReq::npm(req.clone()))
           }
-          PackageJsonDepValue::JsrReq(req) => {
-            // TODO: remove once we support JSR specifiers in package.json
-            log::warn!(
-              "JSR specifiers are not yet supported in package.json: {req}"
-            );
-            None
-          }
           PackageJsonDepValue::Workspace(_) => None,
         })
         .collect()
@@ -344,7 +337,8 @@ impl<TSys: LockfileSys> LockfileLock<TSys> {
       root: WorkspaceMemberConfig {
         package_json_deps: pkg_json_deps(root_folder.pkg_json.as_deref()),
         dependencies: if let Some(map) = maybe_external_import_map {
-          deno_config::import_map::import_map_deps(map)
+          deno_config::import_map::import_map_deps_from_value(map)
+            .collect::<HashSet<_>>()
         } else {
           root_folder
             .deno_json
@@ -405,8 +399,7 @@ impl<TSys: LockfileSys> LockfileLock<TSys> {
                     }
                     // not supported
                     PackageJsonDepValue::File(_)
-                    | PackageJsonDepValue::Workspace(_)
-                    | PackageJsonDepValue::JsrReq(_) => None,
+                    | PackageJsonDepValue::Workspace(_) => None,
                   })
                   .collect()
               })
@@ -461,6 +454,7 @@ impl<TSys: LockfileSys> LockfileLock<TSys> {
           Some((key, value))
         }))
         .collect(),
+      npm_overrides: None,
     };
     lockfile.set_workspace_config(deno_lockfile::SetWorkspaceConfigOptions {
       no_npm: flags.no_npm,
