@@ -3,7 +3,6 @@
 use deno_core::OpState;
 use deno_core::op2;
 use deno_terminal::colors::ColorLevel;
-use serde::Serialize;
 
 use crate::BootstrapOptions;
 
@@ -36,7 +35,7 @@ deno_core::extension!(
   },
 );
 
-#[derive(Serialize)]
+#[derive(serde::Serialize)]
 #[serde(rename_all = "camelCase")]
 pub struct SnapshotOptions {
   pub ts_version: String,
@@ -80,7 +79,6 @@ pub fn op_snapshot_options(state: &mut OpState) -> SnapshotOptions {
 }
 
 #[op2]
-#[serde]
 pub fn op_bootstrap_args(state: &mut OpState) -> Vec<String> {
   state.borrow::<BootstrapOptions>().args.clone()
 }
@@ -104,7 +102,6 @@ pub fn op_bootstrap_user_agent(state: &mut OpState) -> String {
 }
 
 #[op2]
-#[serde]
 pub fn op_bootstrap_unstable_args(state: &mut OpState) -> Vec<String> {
   let options = state.borrow::<BootstrapOptions>();
   let mut flags = Vec::with_capacity(options.unstable_features.len());
